@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* preferences.c - setting the preferences
 
-   Copyright (C) 1998 - 2001 Free Software Foundation
+   Copyright (C) 1998 - 2002 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -23,37 +23,34 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <libgnomeprint/gnome-font-dialog.h>
+
+#include <libgnomeprintui/gnome-font-dialog.h>
 
 #include "ghex.h"
 
-static void set_prefs(PropertyUI *pui);
-static void select_font_cb(GtkWidget *w, GnomePropertyBox *pbox);
-static void apply_changes_cb(GnomePropertyBox *pbox, gint page, PropertyUI *pui);
-static void max_undo_changed_cb(GtkAdjustment *adj, GnomePropertyBox *pbox);
-static void box_size_changed_cb(GtkAdjustment *adj, GnomePropertyBox *pbox);
-static void properties_modified_cb(GtkWidget *w, GnomePropertyBox *pbox);
+static void select_font_cb(GtkWidget *w, const gchar *font_name, GtkDialog *pbox);
+static void apply_changes_cb(PropertyUI *pui);
+static void max_undo_changed_cb(GtkAdjustment *adj, GtkDialog *pbox);
+static void box_size_changed_cb(GtkAdjustment *adj, GtkDialog *pbox);
+static void properties_modified_cb(GtkWidget *w, GtkDialog *pbox);
 static void offset_cb(GtkWidget *w, PropertyUI *pui);
+static void prefs_response_cb(GtkDialog *dlg, gint response, PropertyUI *pui);
 
 PropertyUI *prefs_ui = NULL;
 
-GdkFont *def_font = NULL;
+PangoFontMetrics *def_metrics = NULL; /* Changes for Gnome 2.0 */
+PangoFontDescription *def_font_desc = NULL;
+
 gchar *def_font_name = NULL;
 gboolean show_offsets_column = TRUE;
 
-guint mdi_type[NUM_MDI_MODES] = {
-	GNOME_MDI_DEFAULT_MODE,
-	GNOME_MDI_NOTEBOOK,
-	GNOME_MDI_TOPLEVEL,
-	GNOME_MDI_MODAL
-};
+static char* get_font_name (const gchar *name, int size) {
+	gchar *caption;
 
-gchar *mdi_type_label[NUM_MDI_MODES] = {
-	N_("Default"),
-	N_("Notebook"),
-	N_("Toplevel"),
-	N_("Modal"),
-};
+	g_warning ("Size in get_font_name is %d", size);
+	caption = g_strdup_printf("%s %d", name, size);
+	return caption;
+}
 
 static GtkWidget *get_font_label(const gchar *name, gdouble size) {
 	gchar *caption;
@@ -67,55 +64,62 @@ static GtkWidget *get_font_label(const gchar *name, gdouble size) {
 	return label;
 }
 
-static void font_button_clicked(GtkWidget *button, GnomeFont **font) {
-	static GtkWidget *fsd = NULL;
-	static GtkWidget *fontsel = NULL;
-	const gchar *f_name;
-	gdouble f_size;
-
-	if(!fsd) {
-		fsd = gnome_font_selection_dialog_new(_("Select font"));
-		fontsel = gnome_font_selection_dialog_get_fontsel (GNOME_FONT_SELECTION_DIALOG(fsd));
-		gtk_window_set_modal(GTK_WINDOW(fsd), TRUE);
-		gnome_dialog_close_hides(GNOME_DIALOG(fsd), TRUE);
-	}
-	if(*font)
-		gnome_font_selection_set_font(GNOME_FONT_SELECTION (fontsel),
-									  *font);
-
-	gtk_widget_show(fsd);
-	if(gnome_dialog_run_and_close(GNOME_DIALOG(fsd)) == 0) {
-		GnomeFont *disp_font;
-		disp_font = gnome_font_selection_get_font (GNOME_FONT_SELECTION(fontsel));
-		if(*font)
-			gtk_object_unref(GTK_OBJECT(*font));
-		*font = disp_font;
-		f_name = gnome_font_get_name(*font);
-		f_size = gnome_font_get_size(*font);
-		gtk_container_remove(GTK_CONTAINER(button), GTK_BIN(button)->child);
-		gtk_container_add(GTK_CONTAINER(button), get_font_label(f_name, f_size));
-	}
-}
-
-PropertyUI *create_prefs_dialog() {
+/*
+ * Now we shall have a preferences dialog similar to what gedit2 does.
+ * GnomePropertyBox has been deprecated completely.
+ * We dont need this function. Alteast for now -- SnM
+ * Look at dialogs/ghex-preferences-dialog.[ch]
+ */
+PropertyUI *
+create_prefs_dialog()
+{
 	GtkWidget *vbox, *label, *frame, *box, *entry, *fbox, *flabel, *table;
 	GtkWidget *menu, *item;
 	GtkAdjustment *undo_adj, *box_adj;
-	GnomePaperSelector *paper_sel;
+	GtkWidget *notebook;
+
 	GSList *group;
 	PropertyUI *pui;
-	const gchar *paper_name;
 
 	int i;
 
+	gboolean gail_up;
+
 	pui = g_new0(PropertyUI, 1);
 	
-	pui->pbox = GNOME_PROPERTY_BOX(gnome_property_box_new());
-	
-	gtk_signal_connect(GTK_OBJECT(pui->pbox), "apply",
-					   GTK_SIGNAL_FUNC(apply_changes_cb), pui);
-	gnome_dialog_close_hides(GNOME_DIALOG(pui->pbox), TRUE);
+	pui->pbox = gtk_dialog_new();
+	gtk_window_set_title(GTK_WINDOW(pui->pbox), _("GHex Preferences"));
 	
+	/* create buttons */
+	gtk_dialog_add_button (GTK_DIALOG (pui->pbox),
+						   GTK_STOCK_CANCEL,
+						   GTK_RESPONSE_CANCEL);
+
+	gtk_dialog_add_button (GTK_DIALOG (pui->pbox),
+						   GTK_STOCK_APPLY,
+						   GTK_RESPONSE_APPLY);
+
+	gtk_dialog_add_button (GTK_DIALOG (pui->pbox),
+						   GTK_STOCK_OK,
+						   GTK_RESPONSE_OK);
+
+#if 0
+	gtk_dialog_add_button (GTK_DIALOG (pui->pbox),
+						   GTK_STOCK_HELP,
+						   GTK_RESPONSE_HELP);
+#endif /* 0/1 */
+
+	g_signal_connect(G_OBJECT(pui->pbox), "response",
+					 G_CALLBACK(prefs_response_cb), pui);
+
+	g_signal_connect(G_OBJECT(pui->pbox), "delete-event",
+					 G_CALLBACK (gtk_widget_hide_on_delete),
+					 NULL);
+
+	notebook = gtk_notebook_new();
+	gtk_widget_show(notebook);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(pui->pbox)->vbox), notebook);
+
 	/* editing page */
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(vbox);
@@ -129,22 +133,31 @@ PropertyUI *create_prefs_dialog() {
 	box = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(box);
 
-	label = gtk_label_new(_("Maximum number of undo levels"));
+	label = gtk_label_new_with_mnemonic(_("_Maximum number of undo levels:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX(box), label, TRUE, TRUE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(box), label, TRUE, TRUE, GNOME_PAD);
 	gtk_widget_show(label);
 						  
 	pui->undo_spin = gtk_spin_button_new(undo_adj, 1, 0);
 	gtk_box_pack_end (GTK_BOX(box), GTK_WIDGET(pui->undo_spin), FALSE, TRUE, GNOME_PAD);
 	gtk_widget_show(pui->undo_spin);
 
-	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, TRUE, GNOME_PAD_SMALL);
+	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, TRUE, GNOME_PAD);
 
 	/* cursor offset format */
 	box = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(box);
 
-	label = gtk_label_new(_("Show cursor offset in statusbar as"));
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), pui->undo_spin);
+
+	gail_up = GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(label)) ;
+
+	if (gail_up) {
+		add_atk_namedesc (pui->undo_spin, _("Undo levels"), _("Select maximum number of undo levels"));
+		add_atk_relation (pui->undo_spin, label, ATK_RELATION_LABELLED_BY);
+	}
+
+	label = gtk_label_new(_("Show cursor offset in statusbar as:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX(box), label, TRUE, TRUE, GNOME_PAD);
 	gtk_widget_show(label);
@@ -179,6 +192,17 @@ PropertyUI *create_prefs_dialog() {
 
 	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, TRUE, GNOME_PAD_SMALL);
 
+	if (gail_up) {
+		add_atk_namedesc (pui->format, "format_entry", _("Enter the cursor offset format"));
+		add_atk_namedesc (menu, "format_optionmenu", _("Select the cursor offset format"));
+		add_atk_relation (label, pui->format, ATK_RELATION_LABEL_FOR);
+		add_atk_relation (pui->format, label, ATK_RELATION_LABELLED_BY);
+		add_atk_relation (label, pui->offset_menu, ATK_RELATION_LABEL_FOR);
+		add_atk_relation (pui->offset_menu, label, ATK_RELATION_LABELLED_BY);
+		add_atk_relation (pui->format, pui->offset_menu, ATK_RELATION_CONTROLLED_BY);
+		add_atk_relation (pui->offset_menu, pui->format, ATK_RELATION_CONTROLLER_FOR);
+	}
+
 	/* show offsets check button */
 	pui->offsets_col = gtk_check_button_new_with_label(_("Show offsets column"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->offsets_col), show_offsets_column);
@@ -187,7 +211,7 @@ PropertyUI *create_prefs_dialog() {
 
 	label = gtk_label_new(_("Editing"));
 	gtk_widget_show(label);
-	gtk_notebook_append_page (GTK_NOTEBOOK(pui->pbox->notebook), vbox, label);
+	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 
 	/* display page */
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -198,31 +222,23 @@ PropertyUI *create_prefs_dialog() {
 	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);
 	gtk_widget_show(frame);
 	
-	fbox = gtk_hbox_new(0, 0);
-	entry = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(entry), def_font_name);
-	gtk_signal_connect (GTK_OBJECT (entry), "changed",
-						select_font_cb, pui->pbox);
+	fbox = gtk_hbox_new(0, 5);
 	pui->font_button = gnome_font_picker_new();
 	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(pui->font_button),
 									def_font_name);
 	gnome_font_picker_set_mode(GNOME_FONT_PICKER(pui->font_button),
-							   GNOME_FONT_PICKER_MODE_USER_WIDGET);
+							   GNOME_FONT_PICKER_MODE_FONT_INFO);
+	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (pui->font_button), TRUE, 14);
+	gnome_font_picker_fi_set_show_size (GNOME_FONT_PICKER (pui->font_button), TRUE);
+	
 	gtk_signal_connect (GTK_OBJECT (pui->font_button), "font_set",
 						GTK_SIGNAL_FUNC (select_font_cb), pui->pbox);
-	flabel = gtk_label_new (_("Browse..."));
-	gnome_font_picker_uw_set_widget(GNOME_FONT_PICKER(pui->font_button), GTK_WIDGET(flabel));
-	
-	gtk_object_set_user_data(GTK_OBJECT(entry), GTK_OBJECT(pui->font_button)); 
-	gtk_object_set_user_data (GTK_OBJECT(pui->font_button), GTK_OBJECT(entry)); 
-	
+	flabel = gtk_label_new("");
+	gtk_label_set_mnemonic_widget (GTK_LABEL (flabel), pui->font_button);
 	gtk_widget_show(flabel);
 	gtk_widget_show(GTK_WIDGET(pui->font_button));
-	gtk_widget_show(entry);
-	
 	gtk_container_border_width(GTK_CONTAINER(fbox), GNOME_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX (fbox), entry, 1, 1, GNOME_PAD);
-	gtk_box_pack_end (GTK_BOX (fbox), GTK_WIDGET(pui->font_button), 0, 1, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX (fbox), GTK_WIDGET(pui->font_button), FALSE, TRUE, GNOME_PAD_BIG);
 	
 	gtk_widget_show(fbox);
 	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(fbox));
@@ -248,33 +264,7 @@ PropertyUI *create_prefs_dialog() {
 	
 	label = gtk_label_new(_("Display"));
 	gtk_widget_show(label);
-	gtk_notebook_append_page (GTK_NOTEBOOK(pui->pbox->notebook), vbox, label);
-	
-	/* MDI page */
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_widget_show(vbox);
-
-	/* mdi modes */
-	frame = gtk_frame_new(_("MDI Mode"));
-	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);
-	gtk_widget_show(frame);
-
-	box = gtk_vbox_new(FALSE, 0);
-	gtk_widget_show(box);
-	group = NULL;
-	for(i = 0; i < NUM_MDI_MODES; i++) {
-		pui->mdi_type[i] = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label(group, _(mdi_type_label[i])));
-		gtk_widget_show(GTK_WIDGET(pui->mdi_type[i]));
-		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(pui->mdi_type[i]), TRUE, TRUE, GNOME_PAD_SMALL);
-		group = gtk_radio_button_group (pui->mdi_type[i]);
-	}
-	
-	gtk_container_add(GTK_CONTAINER(frame), box);
-	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, GNOME_PAD_SMALL);  
-	
-	label = gtk_label_new(_("MDI"));
-	gtk_widget_show(label);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pui->pbox->notebook), vbox, label);
+	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 	
 	/* printing page */
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -285,50 +275,81 @@ PropertyUI *create_prefs_dialog() {
 	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);
 	gtk_widget_show(frame);
 
-	pui->paper_sel = gnome_paper_selector_new();
-	paper_sel = GNOME_PAPER_SELECTOR(pui->paper_sel);
-	paper_name = gnome_paper_name(def_paper);
-	gnome_paper_selector_set_name(paper_sel, paper_name);
-	gtk_widget_show(pui->paper_sel);
-	gtk_container_add(GTK_CONTAINER(frame), pui->paper_sel);
-	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE,
-					   GNOME_PAD_SMALL);  
-
 	/* data & header font selection */
 	frame = gtk_frame_new(_("Fonts"));
 	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);
 	gtk_widget_show(frame);
 
-	table = gtk_table_new(2, 2, TRUE);
+	table = gtk_table_new(2, 3, TRUE);
 	gtk_widget_show(table);
-	label = gtk_label_new(_("Data font"));
+	label = gtk_label_new_with_mnemonic(_("_Data font:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
+					 GTK_FILL, GTK_FILL,
 					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-	pui->df_button = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(pui->df_button),
-					  get_font_label(data_font_name, data_font_size));
-	gtk_signal_connect(GTK_OBJECT(pui->df_button), "clicked",
-					   GTK_SIGNAL_FUNC(font_button_clicked), &pui->data_font);
+
+	pui->df_button = gnome_font_picker_new();
+	gnome_font_picker_set_mode (GNOME_FONT_PICKER (pui->df_button),
+								GNOME_FONT_PICKER_MODE_FONT_INFO);
+	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (pui->df_button), TRUE, 14);
+	gnome_font_picker_fi_set_show_size (GNOME_FONT_PICKER (pui->df_button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (pui->df_button), "font_set",
+						GTK_SIGNAL_FUNC (select_font_cb), pui->pbox);
+	pui->df_label = gtk_label_new("");
+	gtk_label_set_mnemonic_widget (GTK_LABEL (pui->df_label), pui->df_button);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), pui->df_button);
+
+	if (gail_up) {
+		add_atk_namedesc (pui->df_button, _("Data font"), _("Select the data font"));
+		add_atk_relation (pui->df_button, label, ATK_RELATION_LABELLED_BY);
+	}	
+	
 	gtk_widget_show(pui->df_button);
 	gtk_table_attach(GTK_TABLE(table), pui->df_button, 1, 2, 0, 1,
 					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
 					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-	label = gtk_label_new(_("Header font"));
+	label = gtk_label_new("");
+	gtk_widget_show(label);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1,
+					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
+					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+
+	label = gtk_label_new_with_mnemonic(_("Header fo_nt:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
+					 GTK_FILL, GTK_FILL,
 					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-	pui->hf_button = gtk_button_new();
-	gtk_signal_connect(GTK_OBJECT(pui->hf_button), "clicked",
-					   GTK_SIGNAL_FUNC(font_button_clicked), &pui->header_font);
+	pui->hf_button = gnome_font_picker_new();
+	gnome_font_picker_set_mode (GNOME_FONT_PICKER (pui->hf_button),
+								GNOME_FONT_PICKER_MODE_FONT_INFO);
+	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (pui->hf_button), TRUE, 14);
+	gnome_font_picker_fi_set_show_size (GNOME_FONT_PICKER (pui->hf_button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (pui->hf_button), "font_set",
+						GTK_SIGNAL_FUNC (select_font_cb), pui->pbox);
+	pui->hf_label = gtk_label_new("");
+	gtk_label_set_mnemonic_widget (GTK_LABEL (pui->hf_label), pui->hf_button);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), pui->hf_button);
+
+	if (gail_up) {
+		add_atk_namedesc (pui->hf_button, _("Header font"), _("Select the header font"));
+		add_atk_relation (pui->hf_button, label, ATK_RELATION_LABELLED_BY);
+	}
+
 	gtk_widget_show(pui->hf_button);
 	gtk_table_attach(GTK_TABLE(table), pui->hf_button, 1, 2, 1, 2,
 					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
 					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+
+	label = gtk_label_new("");
+	gtk_widget_show(label);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 1, 2,
+					 GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL,
+					 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE,
@@ -342,7 +363,7 @@ PropertyUI *create_prefs_dialog() {
 	box = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(box);
 
-	label = gtk_label_new(_("Print shaded box over"));
+	label = gtk_label_new_with_mnemonic(_("_Print shaded box over:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 	gtk_box_pack_start (GTK_BOX(box), label, TRUE, TRUE, GNOME_PAD_SMALL);
 	gtk_widget_show(label);
@@ -351,6 +372,13 @@ PropertyUI *create_prefs_dialog() {
 	gtk_box_pack_start (GTK_BOX(box), GTK_WIDGET(pui->box_size_spin), FALSE, TRUE, GNOME_PAD);
 	gtk_widget_show(pui->box_size_spin);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), pui->box_size_spin);
+
+	if (gail_up) {
+		add_atk_namedesc (pui->box_size_spin, _("Box size"), _("Select size of box (in number of lines)"));
+		add_atk_relation (pui->box_size_spin, label, ATK_RELATION_LABELLED_BY);
+	}
+
 	label = gtk_label_new(_("lines (0 for no box)"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX(box), label, FALSE, TRUE, GNOME_PAD_SMALL);
@@ -360,38 +388,27 @@ PropertyUI *create_prefs_dialog() {
 
 	label = gtk_label_new(_("Printing"));
 	gtk_widget_show(label);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pui->pbox->notebook), vbox, label);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
 
 	set_prefs(pui);
 	
 	/* signals have to be connected after set_prefs(), otherwise
 	   a gnome_property_box_changed() is called */
-   	for(i = 0; i < NUM_MDI_MODES; i++)
-		gtk_signal_connect(GTK_OBJECT(pui->mdi_type[i]), "clicked",
-						   properties_modified_cb, pui->pbox);
-
 	for(i = 0; i < 3; i++)
 		gtk_signal_connect(GTK_OBJECT(pui->group_type[i]), "clicked",
-						   properties_modified_cb, pui->pbox);
+						   GTK_SIGNAL_FUNC(properties_modified_cb), pui->pbox);
 
 	gtk_signal_connect(GTK_OBJECT(pui->offsets_col), "toggled",
-					   properties_modified_cb, pui->pbox);
+					   GTK_SIGNAL_FUNC(properties_modified_cb), pui->pbox);
 
-	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(paper_sel->paper)->entry), "changed",
-					   properties_modified_cb, pui->pbox);
-	gtk_signal_connect(GTK_OBJECT(paper_sel->width), "changed",
-					   properties_modified_cb, pui->pbox);
-	gtk_signal_connect(GTK_OBJECT(paper_sel->height), "changed",
-					   properties_modified_cb, pui->pbox);
-	gtk_signal_connect(GTK_OBJECT(pui->df_button), "clicked",
-					   properties_modified_cb, pui->pbox);
 	gtk_signal_connect(GTK_OBJECT(pui->hf_button), "clicked",
-					   properties_modified_cb, pui->pbox);
+					   GTK_SIGNAL_FUNC(properties_modified_cb), pui->pbox);
 
 	return pui;
 }
 
-static void set_prefs(PropertyUI *pui) {
+
+void set_prefs(PropertyUI *pui) {
 	int i;
 
 	for(i = 0; i < 3; i++)
@@ -400,12 +417,6 @@ static void set_prefs(PropertyUI *pui) {
 			break;
 		}
 	
-	for(i = 0; i < NUM_MDI_MODES; i++)
-		if(mdi_mode == mdi_type[i]) {
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->mdi_type[i]), TRUE);
-			break;
-		}
-
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pui->offsets_col), show_offsets_column);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pui->undo_spin), (gfloat)max_undo_depth);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pui->box_size_spin), (gfloat)shaded_box_size);
@@ -421,71 +432,70 @@ static void set_prefs(PropertyUI *pui) {
 		gtk_widget_set_sensitive(pui->format, TRUE);
 	}
 
-	if(def_paper)
-		gnome_paper_selector_set_name(GNOME_PAPER_SELECTOR(pui->paper_sel),
-									  gnome_paper_name(def_paper));
-	if(GTK_BIN(pui->hf_button)->child)
-		gtk_container_remove(GTK_CONTAINER(pui->hf_button),
-							 GTK_BIN(pui->hf_button)->child);
-	gtk_container_add(GTK_CONTAINER(pui->hf_button),
-					  get_font_label(header_font_name, header_font_size));
-	if(GTK_BIN(pui->df_button)->child)
-		gtk_container_remove(GTK_CONTAINER(pui->df_button),
-							 GTK_BIN(pui->df_button)->child);
-	gtk_container_add(GTK_CONTAINER(pui->df_button),
-					  get_font_label(data_font_name, data_font_size));
-	if(pui->data_font)
-		gtk_object_unref(GTK_OBJECT(pui->data_font));
-	pui->data_font = gnome_font_new(data_font_name, data_font_size);
-	if(pui->header_font)
-		gtk_object_unref(GTK_OBJECT(pui->header_font));
-	pui->header_font = gnome_font_new(header_font_name, header_font_size);	
+	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(pui->hf_button),
+									header_font_name);
+
+	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(pui->df_button),
+									data_font_name);
+
+	gtk_dialog_set_default_response(GTK_DIALOG(pui->pbox), GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(pui->pbox), GTK_RESPONSE_OK, FALSE);
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(pui->pbox), GTK_RESPONSE_APPLY, FALSE);
 }
 
 /*
  * callbacks for properties dialog and its widgets
  */
-static void properties_modified_cb(GtkWidget *w, GnomePropertyBox *pbox) {
-	gnome_property_box_changed(pbox);
+static void properties_modified_cb(GtkWidget *w, GtkDialog *pbox) {
+	gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_OK, TRUE);
+	gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_APPLY, TRUE);
 }
 
-static void max_undo_changed_cb(GtkAdjustment *adj, GnomePropertyBox *pbox) {
-	if((guint)adj->value != max_undo_depth)
-		gnome_property_box_changed(pbox);
+static void max_undo_changed_cb(GtkAdjustment *adj, GtkDialog *pbox) {
+	if((guint)adj->value != max_undo_depth) {
+		gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_OK, TRUE);
+		gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_APPLY, TRUE);
+	}
 }
 
 
-static void box_size_changed_cb(GtkAdjustment *adj, GnomePropertyBox *pbox) {
-	if((guint)adj->value != shaded_box_size)
-		gnome_property_box_changed(pbox);
+static void box_size_changed_cb(GtkAdjustment *adj, GtkDialog *pbox) {
+	if((guint)adj->value != shaded_box_size) {
+		gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_OK, TRUE);
+		gtk_dialog_set_response_sensitive(pbox, GTK_RESPONSE_APPLY, TRUE);
+	}
 }
 
-static void apply_changes_cb(GnomePropertyBox *pbox, gint page, PropertyUI *pui) {
+static void prefs_response_cb(GtkDialog *dlg, gint response, PropertyUI *pui)
+{
+	switch(response) {
+	case GTK_RESPONSE_APPLY:
+		apply_changes_cb(pui);
+		break;
+	case GTK_RESPONSE_OK:
+		apply_changes_cb(pui);
+		gtk_widget_hide(pui->pbox);
+		break;
+	default:
+		gtk_widget_hide(pui->pbox);
+		break;
+	}
+}
+
+static void apply_changes_cb(PropertyUI *pui) {
 	int i, len;
-	GList *child, *view;
-	GdkFont *new_font;
+	const GList *winn, *docn;
+	PangoFontMetrics *new_metrics; /* Changes for Gnome 2.0 */
+	PangoFontDescription *new_desc;
+
 	guint new_undo_max;
 	gboolean show_off, expect_spec;
 	gchar *old_offset_fmt;
-	gchar *new_paper_name;
 
-	if ( page != -1 ) return; /* Only do something on global apply */
-	
 	show_off = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pui->offsets_col));
 	if(show_off != show_offsets_column) {
 		show_offsets_column = show_off;
-		child = mdi->children;
-			
-		while(child) {
-			view = GNOME_MDI_CHILD(child->data)->views;
-			while(view) {
-				gtk_hex_show_offsets(GTK_HEX(view->data), show_off);
-				view = g_list_next(view);
-			}
-			child = g_list_next(child);
-		}
 	}
-		
 
 	for(i = 0; i < 3; i++)
 		if(GTK_TOGGLE_BUTTON(pui->group_type[i])->active) {
@@ -493,24 +503,9 @@ static void apply_changes_cb(GnomePropertyBox *pbox, gint page, PropertyUI *pui)
 			break;
 		}
 	
-	for(i = 0; i < NUM_MDI_MODES; i++)
-		if(GTK_TOGGLE_BUTTON(pui->mdi_type[i])->active) {
-			if(mdi_mode != mdi_type[i]) {
-				mdi_mode = mdi_type[i];
-				gnome_mdi_set_mode(mdi, mdi_mode);
-			}
-			break;
-		}
-
 	new_undo_max = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pui->undo_spin));
 	if(new_undo_max != max_undo_depth) {
 		max_undo_depth = new_undo_max;
-
-		child = mdi->children;
-		while(child) {
-			hex_document_set_max_undo(HEX_DOCUMENT(child->data), max_undo_depth);
-			child = child->next;
-		}
 	}
 
 	shaded_box_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pui->box_size_spin));
@@ -532,7 +527,7 @@ static void apply_changes_cb(GnomePropertyBox *pbox, gint page, PropertyUI *pui)
 			   offset_fmt[i] != 'P' && offset_fmt[i] != 'p') {
 				g_free(offset_fmt);
 				offset_fmt = old_offset_fmt;
-				gtk_entry_select_region(GTK_ENTRY(pui->format), i, i+1);
+				gtk_entry_set_text(GTK_ENTRY(pui->format), old_offset_fmt);
 				gnome_error_dialog_parented(_("The offset format string contains invalid format specifier.\n"
 											  "Only 'x', 'X', 'p', 'P', 'd' and 'o' are allowed."),
 											GTK_WINDOW(pui->pbox));
@@ -546,65 +541,40 @@ static void apply_changes_cb(GnomePropertyBox *pbox, gint page, PropertyUI *pui)
 
 	if(strcmp(gnome_font_picker_get_font_name(GNOME_FONT_PICKER
 											  (pui->font_button)), def_font_name) != 0) {
-		if((new_font = gdk_font_load(gnome_font_picker_get_font_name
+		if((new_metrics = gtk_hex_load_font(gnome_font_picker_get_font_name
 									 (GNOME_FONT_PICKER(pui->font_button)))) != NULL) {
-			child = mdi->children;
+			new_desc = pango_font_description_from_string (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (pui->font_button)));
+			if (def_metrics)
+				pango_font_metrics_unref (def_metrics);
 			
-			while(child) {
-				view = GNOME_MDI_CHILD(child->data)->views;
-				while(view) {
-					gtk_hex_set_font(GTK_HEX(view->data), new_font);
-					view = g_list_next(view);
-				}
-				child = g_list_next(child);
-			}
-			
-			if(def_font)
-				gdk_font_unref(def_font);
-			
-			def_font = new_font;
-			
-			g_free(def_font_name);
+			if (def_font_desc)
+				pango_font_description_free (def_font_desc);
+
+			def_metrics = new_metrics;
 			
+			if(def_font_name)
+				g_free(def_font_name);
+
 			def_font_name = g_strdup(gnome_font_picker_get_font_name
 									 (GNOME_FONT_PICKER(pui->font_button)));
+			def_font_desc = new_desc;
 		}
 		else
-			gnome_app_error(mdi->active_window, _("Can not open desired font!"));
-	} 
-
-	new_paper_name = gnome_paper_selector_get_name(GNOME_PAPER_SELECTOR(pui->paper_sel));
-	def_paper = gnome_paper_with_name(new_paper_name);
-
-	if(pui->data_font) {
-		if(data_font_name)
-			g_free(data_font_name);
-		data_font_name = g_strdup(gnome_font_get_name(GNOME_FONT(pui->data_font)));
-		data_font_size = gnome_font_get_size(GNOME_FONT(pui->data_font));
-	}
-	if(pui->header_font) {
-		if(header_font_name)
-			g_free(header_font_name);
-		header_font_name = g_strdup(gnome_font_get_name(GNOME_FONT(pui->header_font)));
-		header_font_size = gnome_font_get_size(GNOME_FONT(pui->header_font));
+			display_error_dialog (ghex_window_get_active(), _("Can not open desired font!"));
 	}
+
+	if(data_font_name)
+		g_free(data_font_name);
+	data_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (pui->df_button)));
+	if(header_font_name)
+		g_free(header_font_name);
+	header_font_name = g_strdup (gnome_font_picker_get_font_name (GNOME_FONT_PICKER (pui->hf_button)));
+
+	save_configuration();
 }
 
-static void select_font_cb(GtkWidget *w, GnomePropertyBox *pbox) {
-	gchar *font_desc;
-	GtkWidget *peer;
-
-	if (GNOME_IS_FONT_PICKER(w)) {
-		font_desc = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(w));
-		peer = gtk_object_get_user_data (GTK_OBJECT(w));
-		gtk_entry_set_text (GTK_ENTRY(peer), font_desc);
-	}
-	else {
-		font_desc = gtk_entry_get_text (GTK_ENTRY(w));
-		peer = gtk_object_get_user_data (GTK_OBJECT(w));
-		gnome_font_picker_set_font_name (GNOME_FONT_PICKER(peer), font_desc);
-		gnome_property_box_changed(pbox);
-	}
+static void select_font_cb(GtkWidget *w, const gchar *font_name, GtkDialog *pbox) {
+	properties_modified_cb(w, pbox);
 }
 
 static void offset_cb(GtkWidget *w, PropertyUI *pui) {
@@ -628,5 +598,5 @@ static void offset_cb(GtkWidget *w, PropertyUI *pui) {
 		break;
 	}
 
-	properties_modified_cb(w, pui->pbox);	
+	properties_modified_cb(w, GTK_DIALOG(pui->pbox));
 }
