@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* print.c - print a HexDocument
 
-   Copyright (C) 1998 - 2001 Free Software Foundation
+   Copyright (C) 1998 - 2002 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -24,11 +24,17 @@
 
 #include "ghex.h"
 #include "gtkhex.h"
-#include <libgnomeprint/gnome-print-master-preview.h>
+#include <libgnomeprintui/gnome-print-dialog.h> /* Added for Gnome 2 -- SnM */
 
 #define is_printable(c) (((((guchar)c)>=0x20) && (((guchar)c)<=0x7F))?1:0)
 
-const GnomePaper *def_paper;
+/* Taken from gedit SnM */
+#define CM(v) ((v) * 72.0 / 2.54)
+#define A4_WIDTH (210.0 * 72 / 25.4)
+#define A4_HEIGHT (297.0 * 72 / 25.4)
+
+const guchar *def_paper; /* Using guchar instead -- SnM */
+
 gchar *data_font_name, *header_font_name;
 gdouble data_font_size, header_font_size;
 gint shaded_box_size;
@@ -46,27 +52,31 @@ static void print_header(GHexPrintJobInfo *pji, unsigned int page)
 {
 	guchar* text1 = g_strdup(pji->doc->file_name);
 	guchar* text2 = g_strdup_printf(_("Page: %i/%i"),page,pji->pages);
+	guchar* pagetext = g_strdup_printf("%d", page);
 	gfloat x, y, len;
 	
+	gnome_print_beginpage(pji->pc, pagetext);
 	gnome_print_setfont(pji->pc, pji->h_font);
 	/* Print the file name */
 	y = pji->page_height - pji->margin_top -
 		2.1*gnome_font_get_ascender(pji->h_font) -
 		1.1*gnome_font_get_descender(pji->h_font);
-	len = gnome_font_get_width_string (pji->h_font, text1);
+	len = gnome_font_get_width_utf8 (pji->h_font, text1);
 	x = pji->page_width/2 - len/2;
 	gnome_print_moveto(pji->pc, x, y);
 	gnome_print_show(pji->pc, text1);
-	gnome_print_stroke(pji->pc);
 
 	/* Print the page/pages  */
 	y = pji->page_height - pji->margin_top -
 		gnome_font_get_ascender(pji->h_font);
-	len = gnome_font_get_width_string (pji->h_font, text2);
+	len = gnome_font_get_width_utf8 (pji->h_font, text2);
 	x = pji->page_width - len - 36;
 	gnome_print_moveto(pji->pc, x, y);
 	gnome_print_show(pji->pc, text2);
-	gnome_print_stroke(pji->pc); 
+
+	g_free(text1);
+	g_free(text2);
+	g_free(pagetext);
 }
 
 #define TEMP_LEN 256
@@ -85,7 +95,6 @@ static void print_row(GHexPrintJobInfo *pji, unsigned int offset, unsigned int b
 	gnome_print_moveto(pji->pc, x , y);
 	g_snprintf(temp, TEMP_LEN, "%08X", offset);
 	gnome_print_show(pji->pc, temp);
-	gnome_print_stroke(pji->pc);
 	/* Print Hex */
 	x = pji->margin_left +
 		pji->font_char_width*pji->offset_chars +
@@ -93,14 +102,12 @@ static void print_row(GHexPrintJobInfo *pji, unsigned int offset, unsigned int b
 	gnome_print_moveto(pji->pc, x, y);
 	format_hex(pji->doc, pji->gt, temp, offset, offset + bytes);
 	gnome_print_show(pji->pc, temp);
-	gnome_print_stroke(pji->pc);
 	/* Print Ascii */
 	x = 2*pji->pad_size + pji->margin_left + pji->font_char_width*
 		(pji->offset_chars + pji->bytes_per_row*(2 + (1/((float)pji->gt))));
 	gnome_print_moveto(pji->pc, x, y);
 	format_ascii(pji->doc, temp, offset, offset + bytes);
 	gnome_print_show(pji->pc, temp);
-	gnome_print_stroke(pji->pc);
 
 	g_free(temp);
 }
@@ -192,13 +199,13 @@ static gboolean print_verify_fonts()
 	guchar *test_font_name;
 
 	test_font_name = g_strdup(data_font_name); 
-	test_font = gnome_font_new(test_font_name, 10);
+	test_font = gnome_font_find_closest_from_full_name(test_font_name);
 	if(test_font==NULL)
 	{
 		gchar *errstr = g_strdup_printf(_("GHex could not find the font \"%s\".\n"
 										  "GHex is unable to print without this font installed."),
 										test_font_name);
-		gnome_app_error(mdi->active_window, errstr);
+		display_error_dialog (ghex_window_get_active(), errstr);
 		g_free(errstr);
 		return FALSE;
 	}
@@ -206,22 +213,73 @@ static gboolean print_verify_fonts()
 	g_free(test_font_name);
 	
 	test_font_name = g_strdup(header_font_name);
-	test_font = gnome_font_new(test_font_name, 10);
+	test_font = gnome_font_find_closest_from_full_name(test_font_name);
 	if(test_font==NULL)
 	{
 		gchar *errstr = g_strdup_printf(_("GHex could not find the font \"%s\".\n"
 										  "GHex is unable to print without this font installed."),
 										test_font_name);
-		gnome_app_error(mdi->active_window, errstr);
+		display_error_dialog (ghex_window_get_active(), errstr);
 		g_free(errstr);
 		return FALSE;
 	}
 	gnome_font_unref(test_font);
 	g_free(test_font_name);	
- 
 	return TRUE;
 }
 
+void
+ghex_print_update_page_size_and_margins (HexDocument *doc, GHexPrintJobInfo *pji)
+{
+	const GnomePrintUnit *unit;
+
+	gnome_print_master_get_page_size_from_config (pji->config,
+			&pji->page_width, &pji->page_height);
+
+	if (gnome_print_config_get_length (pji->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT,
+				&pji->margin_left, &unit))
+	{
+		gnome_print_convert_distance (&pji->margin_left, unit, GNOME_PRINT_PS_UNIT);
+	}
+
+	if (gnome_print_config_get_length (pji->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT,
+				&pji->margin_right, &unit))
+	{
+		gnome_print_convert_distance (&pji->margin_right, unit, GNOME_PRINT_PS_UNIT);
+	}
+
+	if (gnome_print_config_get_length (pji->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP,
+				&pji->margin_top, &unit))
+	{
+		gnome_print_convert_distance (&pji->margin_top, unit, GNOME_PRINT_PS_UNIT);
+	}
+	if (gnome_print_config_get_length (pji->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM,
+				&pji->margin_bottom, &unit))
+	{
+		gnome_print_convert_distance (&pji->margin_bottom, unit, GNOME_PRINT_PS_UNIT);
+	}
+
+	pji->printable_width = pji->page_width -
+		pji->margin_left -
+		pji->margin_right;
+	pji->printable_height = pji->page_height -
+		pji->margin_top -
+		pji->margin_bottom;
+
+	pji->bytes_per_row = (pji->printable_width - pji->pad_size*2 -
+						(pji->offset_chars *
+						 pji->font_char_width))/
+				((3 + (1/((float)pji->gt))) *
+				 pji->font_char_width);
+	pji->bytes_per_row -= pji->bytes_per_row % pji->gt;
+	pji->rows_per_page = (pji->printable_height - pji->header_height) /
+		pji->font_char_height - 1;
+	pji->pages = (((doc->file_size/pji->bytes_per_row) + 1)/
+				pji->rows_per_page) + 1;
+	pji->page_first = 1;
+	pji->page_last = pji->pages;
+}
+
 /**
  * ghex_print_job_info_new:
  * @doc: Pointer to the HexDocument to be printed.
@@ -245,11 +303,11 @@ ghex_print_job_info_new(HexDocument *doc, guint group_type)
 		return NULL;
 
 	/* Create the header and data fonts */
-	d_font = gnome_font_new(data_font_name, data_font_size);
+	d_font = gnome_font_find_closest_from_full_name(data_font_name);
 	if (!d_font)
 		return NULL;
 
-	h_font = gnome_font_new(header_font_name, header_font_size);
+	h_font = gnome_font_find_closest_from_full_name(header_font_name);
 	if (!h_font) {
 		gnome_font_unref(d_font);
 		return NULL;
@@ -260,57 +318,33 @@ ghex_print_job_info_new(HexDocument *doc, guint group_type)
 	pji->d_font = d_font;
 	pji->gt = group_type;
 
-	pji->master = gnome_print_master_new();
-
-	gnome_print_master_set_paper(pji->master, def_paper);
+	pji->master = NULL;
 
 	pji->doc = doc;
 
-	pji->page_width = gnome_paper_pswidth(def_paper);
-	pji->page_height = gnome_paper_psheight(def_paper);
+	pji->page_width = A4_WIDTH;
+	pji->page_height = A4_HEIGHT;
 
-	/* Convert inches to ps points */
-	pji->margin_top = .75 * 72; /* Printer margins, not page margins */
-	pji->margin_bottom = .75 * 72;
-	pji->margin_left = .75 * 72;
-	pji->margin_right = .75 * 72;
-	pji->header_height = 2.2*(gnome_font_get_ascender(GNOME_FONT(h_font)) +
-		gnome_font_get_descender(GNOME_FONT(h_font)));
+	pji->margin_top = CM (1);
+	pji->margin_bottom = CM (1);
+	pji->margin_left = CM (1);
+	pji->margin_right = CM (1);
+	pji->header_height = 2.5 * gnome_font_get_size (pji->h_font);
 
 	/* Get font character width */
 	glyph = ' ';
 	gnome_font_get_glyph_stdadvance(GNOME_FONT(d_font), glyph, &point);
 	pji->font_char_width = point.x;
 
-	pji->font_char_height = gnome_font_get_ascender(GNOME_FONT(d_font)) +
-		gnome_font_get_descender(GNOME_FONT(d_font));
+	pji->font_char_height = gnome_font_get_size (GNOME_FONT(d_font));
 
 	/* Add 10% spacing between lines */
 	pji->font_char_height *= 1.1;
 	pji->pad_size = .5 * 72;
 	pji->offset_chars = 8;
 
-	pji->printable_width = pji->page_width -
-		pji->margin_left -
-		pji->margin_right;
-	pji->printable_height = pji->page_height -
-		pji->margin_top -
-		pji->margin_bottom;
-
-	pji->bytes_per_row = (pji->printable_width - pji->pad_size*2 -
-						(pji->offset_chars *
-						 pji->font_char_width))/
-				((3 + (1/((float)pji->gt))) *
-				 pji->font_char_width);
-	pji->bytes_per_row -= pji->bytes_per_row % pji->gt;
-	pji->rows_per_page = (pji->printable_height - pji->header_height) /
-		pji->font_char_height - 1;
-	pji->pages = (((doc->file_size/pji->bytes_per_row) + 1)/
-				pji->rows_per_page) + 1;
-	pji->page_first = 1;
-	pji->page_last = pji->pages;
 	pji->preview = FALSE;
-	pji->printer = NULL;
+	pji->config = NULL;
 	pji->range = GNOME_PRINT_RANGE_ALL;
 
 	return pji;
@@ -328,6 +362,16 @@ ghex_print_job_info_destroy(GHexPrintJobInfo *pji)
 	gnome_print_master_close(pji->master);
 	gnome_font_unref(pji->h_font);
 	gnome_font_unref(pji->d_font);
+
+	if (pji->config != NULL)
+		gnome_print_config_unref (pji->config);
+
+	if (pji->pc != NULL)
+		g_object_unref (pji->pc);
+
+	if (pji->master != NULL)
+		g_object_unref (pji->master);
+
 	g_free(pji);
 }
 
@@ -375,5 +419,5 @@ ghex_print_job_execute(GHexPrintJobInfo *pji)
 		}
 		end_page(pji->pc);
 	}
-	gnome_print_context_close(pji->pc);
 }
+
